<?php

namespace Drupal\flush\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\file\Entity\File;

/**
 * Configure settings for this module.
 */
class FlushSettingsForm extends ConfigFormBase {

  /**
   * Config settings.
   *
   * @var string
   */
  const SETTINGS = 'flush.settings';

  /**
   * Path to flush asset.
   *
   * @var string
   */
  const PATH = '/assets/flush.mp3';

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'flush_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::SETTINGS,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config(static::SETTINGS);

    $form['pathToSound'] = array(
      '#markup' => '<p> Current sound path: ' . $config->get('pathToSound') . '</p>',
    );
    // Sets pathToSound to default sound that comes with Flush.
    $form['set_default'] = array(
      '#type' => 'submit',
      '#value' => $this->t('Set to Default (Toilet Flush)'),
      '#submit' => array([$this, 'setToDefault']),
    );
    $form['or'] = array(
      '#markup' => '<p> or </p>',
    );
    // Allows user to upload their own sound and sets pathToSound to that.
    $form['upload']['sound_dir'] = [
      '#type'                 => 'managed_file',
      '#upload_location'      => 'public://',
      '#required'             => FALSE,
      '#multiple'             => FALSE,
      '#description'          => $this->t('Allowed extensions: mp3 wav'),
      '#upload_validators'    => [
        'file_validate_extensions'    => array('mp3 wav'),
        'file_validate_size'          => array(25600000)
      ],
      '#title'                => $this->t('Upload a sound file:')
      ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function setToDefault(array &$form, FormStateInterface $form_state){
    $url = $this->getRequest()->getSchemeAndHttpHost();
    $url = $url.\Drupal::request()->getBaseUrl();
    $module_handler = \Drupal::service('module_handler');
    $module_path = $module_handler->getModule('flush')->getPath();
    $url = $url.'/'.$module_path.static::PATH;

    $this->configFactory->getEditable(static::SETTINGS)
      ->set('pathToSound', $url)
      ->set('useDefault', true)
      ->save();
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $file_data = $form_state->getValue(['upload' => 'sound_dir']);
    // If the submit button is pressed and the user actually uploaded an image,
    // upload it and set the url to its directory.
    if ($file_data) {
      $file = File::load( $file_data[0] );
      $file_name = $file->getFilename();
      $uri = $file->getFileUri();
      $url = Url::fromUri(file_create_url($uri))->toString();
      $file->setPermanent();
      $file->save();

      $this->configFactory->getEditable(static::SETTINGS)
        ->set('pathToSound', $url)
        ->set('useDefault', false)
        ->save();
      parent::submitForm($form, $form_state);
    }
  }
}
